(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var SandstormDb = Package['sandstorm-db'].SandstormDb;

/* Package-scope variables */
var SandstormPermissions;

(function () {

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
// packages/sandstorm-permissions/permissions.js                                                      //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                      //
// Sandstorm - Personal Cloud Sandbox                                                                 // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                              // 2
// All rights reserved.                                                                               // 3
//                                                                                                    // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                    // 5
// you may not use this file except in compliance with the License.                                   // 6
// You may obtain a copy of the License at                                                            // 7
//                                                                                                    // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                       // 9
//                                                                                                    // 10
// Unless required by applicable law or agreed to in writing, software                                // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                  // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                           // 13
// See the License for the specific language governing permissions and                                // 14
// limitations under the License.                                                                     // 15
                                                                                                      // 16
var Crypto = Npm.require("crypto");                                                                   // 17
var Url = Npm.require("url");                                                                         // 18
                                                                                                      // 19
SandstormPermissions = {};                                                                            // 20
                                                                                                      // 21
function PermissionSet(array) {                                                                       // 22
  // A wrapper around an array of booleans.                                                           // 23
                                                                                                      // 24
  if (!array) {                                                                                       // 25
    this.array = [];                                                                                  // 26
  } else if (array instanceof Array) {                                                                // 27
    this.array = array.slice(0);                                                                      // 28
  } else {                                                                                            // 29
    throw new Error("don't know how to interpret as PermissionSet: " + array);                        // 30
  }                                                                                                   // 31
}                                                                                                     // 32
                                                                                                      // 33
// Methods for mutating a PermissionSet by combining it with another PermissionSet.                   // 34
// These return a boolean indicating whether the operation had any effect.                            // 35
                                                                                                      // 36
PermissionSet.prototype.add = function(other) {                                                       // 37
  var changed = false;                                                                                // 38
  for (var ii = 0; ii < other.array.length; ++ii) {                                                   // 39
    var old = this.array[ii];                                                                         // 40
    this.array[ii] = this.array[ii] || other.array[ii];                                               // 41
    if (old != this.array[ii]) {                                                                      // 42
      changed = true;                                                                                 // 43
    }                                                                                                 // 44
  }                                                                                                   // 45
  return changed;                                                                                     // 46
}                                                                                                     // 47
                                                                                                      // 48
PermissionSet.prototype.remove = function(other) {                                                    // 49
  var changed = false;                                                                                // 50
  for (var ii = 0; ii < other.array.length && ii < this.array.length; ++ii) {                         // 51
    var old = this.array[ii];                                                                         // 52
    this.array[ii] = this.array[ii] && !other.array[ii];                                              // 53
    if (old != this.array[ii]) {                                                                      // 54
      changed = true;                                                                                 // 55
    }                                                                                                 // 56
  }                                                                                                   // 57
  return changed;                                                                                     // 58
}                                                                                                     // 59
                                                                                                      // 60
PermissionSet.prototype.intersect = function(other) {                                                 // 61
  var changed = false;                                                                                // 62
  for (var ii = 0; ii < this.array.length; ++ii) {                                                    // 63
    var old = this.array[ii];                                                                         // 64
    this.array[ii] = this.array[ii] && other.array[ii];                                               // 65
    if (old != this.array[ii]) {                                                                      // 66
      changed = true;                                                                                 // 67
    }                                                                                                 // 68
  }                                                                                                   // 69
  return changed;                                                                                     // 70
}                                                                                                     // 71
                                                                                                      // 72
function roleAssignmentPermissions(roleAssignment, viewInfo) {                                        // 73
  var result = new PermissionSet([]);                                                                 // 74
                                                                                                      // 75
  if (!roleAssignment || "none" in roleAssignment) {                                                  // 76
    if (viewInfo.roles) {                                                                             // 77
      for (var ii = 0; ii < viewInfo.roles.length; ++ii) {                                            // 78
        var roleDef = viewInfo.roles[ii];                                                             // 79
        if (roleDef.default) {                                                                        // 80
          result = new PermissionSet(roleDef.permissions);                                            // 81
          break;                                                                                      // 82
        }                                                                                             // 83
      }                                                                                               // 84
    }                                                                                                 // 85
  } else if ("allAccess" in roleAssignment) {                                                         // 86
    var length = 0;                                                                                   // 87
    if (viewInfo.permissions) {                                                                       // 88
      length = viewInfo.permissions.length;                                                           // 89
    }                                                                                                 // 90
    var array = new Array(length);                                                                    // 91
    for (var ii = 0; ii < array.length; ++ii) {                                                       // 92
      array[ii] = true;                                                                               // 93
    }                                                                                                 // 94
    result = new PermissionSet(array);                                                                // 95
  } else if ("roleId" in roleAssignment && viewInfo.roles && viewInfo.roles.length > 0) {             // 96
    var roleDef = viewInfo.roles[roleAssignment.roleId];                                              // 97
    if (roleDef) {                                                                                    // 98
      result = new PermissionSet(roleDef.permissions);                                                // 99
    }                                                                                                 // 100
  }                                                                                                   // 101
                                                                                                      // 102
  if (roleAssignment) {                                                                               // 103
    result.add(new PermissionSet(roleAssignment.addPermissionSet));                                   // 104
    result.remove(new PermissionSet(roleAssignment.removePermissionSet));                             // 105
  }                                                                                                   // 106
  return result;                                                                                      // 107
}                                                                                                     // 108
                                                                                                      // 109
function collectEdges(db, vertex) {                                                                   // 110
  // Given a vertex in the sharing graph in the format specified by the `check()` invocation below,   // 111
  // collects the data needed for permissions computations pertaining to that vertex. There are four  // 112
  // self-explanatory special cases for the return value. In order of decreasing precedence, they     // 113
  // are: `{grainDoesNotExist: true}`, `{openerIsOwner: true}`, `{grainIsPublic: true}`, and          // 114
  // `{disallowedAnonymousAccess: true}`. In all other cases, this function returns an object         // 115
  // of the form: `{owner: <userId>, edgesByRecipient: <object>, terminalEdge: Maybe(<object>)}`.     // 116
  // The `owner` field indicates the user who owns the grain. The `edgesByRecipient` field is a map   // 117
  // that coalesces chains of `parentToken` UiView tokens into direct user-to-user edges; its keys    // 118
  // are IDs of recipient users and its values are lists of "edge" objects of the form                // 119
  // `{sharer: <userId>, roleAssignments: <list of role assignments>}`.  The role assignments         // 120
  // should be applied in sequence to compute the set of permissions that flow to a recipient from    // 121
  // a sharer. The `terminalEdge` field is an edge object representing the link to `vertex` from      // 122
  // the nearest user in the sharing graph. If `vertex` is already a user, then this edge is          // 123
  // trivial and its `roleAssignments` field is an empty list. If `terminalEdge` is not present,      // 124
  // then there is no such link.                                                                      // 125
  //                                                                                                  // 126
  // TODO(someday): Once UiView tokens can have membrane requirements, we'll need to account for      // 127
  // them in this computation.                                                                        // 128
  check(vertex,                                                                                       // 129
        Match.OneOf({token: Match.ObjectIncluding({_id: String, grainId: String})},                   // 130
                    {grain: Match.ObjectIncluding({_id: String,                                       // 131
                                                   userId: Match.OneOf(String, null, undefined)})})); // 132
                                                                                                      // 133
  var grainId;                                                                                        // 134
  if (vertex.token) {                                                                                 // 135
    grainId = vertex.token.grainId;                                                                   // 136
  } else if (vertex.grain) {                                                                          // 137
    grainId = vertex.grain._id;                                                                       // 138
  }                                                                                                   // 139
  var grain = db.getGrain(grainId);                                                                   // 140
  if (!grain) {                                                                                       // 141
    return {grainDoesNotExist: true};                                                                 // 142
  }                                                                                                   // 143
                                                                                                      // 144
  if (vertex.grain && grain.userId === vertex.grain.userId) {                                         // 145
    return {openerIsOwner: true};                                                                     // 146
  }                                                                                                   // 147
                                                                                                      // 148
  if (!grain.private) {                                                                               // 149
    return {grainIsPublic: true};                                                                     // 150
  } else if (vertex.grain && !vertex.grain.userId) {                                                  // 151
    return {disallowedAnonymousAccess: true};                                                         // 152
  }                                                                                                   // 153
                                                                                                      // 154
  var result = {edgesByRecipient: {}, owner: grain.userId};                                           // 155
  var tokensById = {};                                                                                // 156
  db.collections.apiTokens.find({grainId: grainId,                                                    // 157
                                 revoked: {$ne: true}}).forEach(function(token) {                     // 158
    tokensById[token._id] = token;                                                                    // 159
  });                                                                                                 // 160
                                                                                                      // 161
  function computeEdge(token) {                                                                       // 162
    var roleAssignments = [];                                                                         // 163
    var curToken = token;                                                                             // 164
    while (curToken && curToken.parentToken) {                                                        // 165
      // For a `parentToken` provider, no role assignment means don't attenuate.                      // 166
      if (curToken.roleAssignment) {                                                                  // 167
        roleAssignments.push(curToken.roleAssignment);                                                // 168
      }                                                                                               // 169
      curToken = tokensById[curToken.parentToken];                                                    // 170
    }                                                                                                 // 171
    if (curToken && curToken.grainId && curToken.userId) {                                            // 172
      roleAssignments.push(curToken.roleAssignment);                                                  // 173
      return {sharer: curToken.userId, roleAssignments: roleAssignments};                             // 174
    } else {                                                                                          // 175
      return;                                                                                         // 176
    }                                                                                                 // 177
  }                                                                                                   // 178
                                                                                                      // 179
  if (vertex.token) {                                                                                 // 180
    result.terminalEdge = computeEdge(tokensById[vertex.token._id]);                                  // 181
  } else if (vertex.grain) {                                                                          // 182
    result.terminalEdge = {sharer: vertex.grain.userId, roleAssignments: []};                         // 183
  }                                                                                                   // 184
                                                                                                      // 185
  for (var id in tokensById) {                                                                        // 186
    var token = tokensById[id];                                                                       // 187
    if (Match.test(token.owner, {user: Match.Any})) {                                                 // 188
      var edge = computeEdge(token);                                                                  // 189
      if (edge) {                                                                                     // 190
        var recipient = token.owner.user.userId ;                                                     // 191
        if (!result.edgesByRecipient[recipient]) {                                                    // 192
          result.edgesByRecipient[recipient] = [];                                                    // 193
        }                                                                                             // 194
        result.edgesByRecipient[recipient].push(edge);                                                // 195
      }                                                                                               // 196
    }                                                                                                 // 197
  }                                                                                                   // 198
  return result;                                                                                      // 199
}                                                                                                     // 200
                                                                                                      // 201
SandstormPermissions.grainPermissions = function(db, vertex, viewInfo) {                              // 202
  // Computes the permissions of a vertex. If the vertex is not allowed to open the grain,            // 203
  // returns null. Otherwise, returns an array of bools representing the permissions held.            // 204
  var edges = collectEdges(db, vertex);                                                               // 205
  if (edges.openerIsOwner) {                                                                          // 206
    return roleAssignmentPermissions({allAccess: null}, viewInfo).array;                              // 207
  }                                                                                                   // 208
  if (edges.grainIsPublic) {                                                                          // 209
    // Grains using the old sharing model always share the default role to anyone who has the         // 210
    // grain URL.                                                                                     // 211
    return roleAssignmentPermissions({none: null}, viewInfo).array;                                   // 212
  }                                                                                                   // 213
  if (edges.grainDoesNotExist || !edges.terminalEdge || edges.disallowedAnonymousAccess) {            // 214
    return null;                                                                                      // 215
  }                                                                                                   // 216
                                                                                                      // 217
  var openerUserId = edges.terminalEdge.sharer;                                                       // 218
  var edgesByRecipient = edges.edgesByRecipient;                                                      // 219
  var owner = edges.owner;                                                                            // 220
                                                                                                      // 221
  var permissionsMap = {};                                                                            // 222
  // Keeps track of the permissions that the opener receives from each user. The final result of      // 223
  // our computation will be stored in permissionsMap[owner].                                         // 224
                                                                                                      // 225
  var userStack = [openerUserId];                                                                     // 226
                                                                                                      // 227
  var openerAttenuation = roleAssignmentPermissions({allAccess: null}, viewInfo);                     // 228
  edges.terminalEdge.roleAssignments.forEach(function(roleAssignment) {                               // 229
    openerAttenuation.intersect(roleAssignmentPermissions(roleAssignment, viewInfo));                 // 230
  });                                                                                                 // 231
  permissionsMap[openerUserId] = openerAttenuation;                                                   // 232
                                                                                                      // 233
  while (userStack.length > 0) {                                                                      // 234
    var recipient = userStack.pop();                                                                  // 235
    if (edgesByRecipient[recipient]) {                                                                // 236
      edgesByRecipient[recipient].forEach(function (inEdge) {                                         // 237
        var sharer = inEdge.sharer;                                                                   // 238
        var needToPush = false;                                                                       // 239
        if (!permissionsMap[sharer]) {                                                                // 240
          permissionsMap[sharer] = new PermissionSet();                                               // 241
          needToPush = true;                                                                          // 242
        }                                                                                             // 243
                                                                                                      // 244
        var newPermissions = roleAssignmentPermissions({allAccess: null}, viewInfo);                  // 245
        inEdge.roleAssignments.forEach(function(roleAssignment) {                                     // 246
          newPermissions.intersect(roleAssignmentPermissions(roleAssignment, viewInfo));              // 247
        });                                                                                           // 248
        newPermissions.intersect(permissionsMap[recipient]);                                          // 249
                                                                                                      // 250
        // Optimization: we don't care about permissions that we've already proven the opener has.    // 251
        if (permissionsMap[owner]) {                                                                  // 252
          newPermissions.remove(permissionsMap[owner]);                                               // 253
        }                                                                                             // 254
                                                                                                      // 255
        if (permissionsMap[sharer].add(newPermissions)) {                                             // 256
          needToPush = true;                                                                          // 257
        }                                                                                             // 258
        if (needToPush) {                                                                             // 259
          userStack.push(sharer);                                                                     // 260
        }                                                                                             // 261
      });                                                                                             // 262
    }                                                                                                 // 263
  }                                                                                                   // 264
  if (permissionsMap[owner]) {                                                                        // 265
    return permissionsMap[owner].array;                                                               // 266
  } else {                                                                                            // 267
    return null;                                                                                      // 268
  }                                                                                                   // 269
}                                                                                                     // 270
                                                                                                      // 271
SandstormPermissions.mayOpenGrain = function(db, vertex) {                                            // 272
  // Determines whether the vertex is allowed to open the grain by searching depth first              // 273
  // for a path of active role assignments leading from the grain owner to the vertex.                // 274
                                                                                                      // 275
  var edges = collectEdges(db, vertex);                                                               // 276
  if (edges.grainDoesNotExist || edges.disallowedAnonymousAccess) {                                   // 277
    return false;                                                                                     // 278
  }                                                                                                   // 279
  if (edges.openerIsOwner || edges.grainIsPublic) {                                                   // 280
    return true;                                                                                      // 281
  }                                                                                                   // 282
  var edgesByRecipient = edges.edgesByRecipient;                                                      // 283
  var owner = edges.owner;                                                                            // 284
  if (!edges.terminalEdge) { return false; }                                                          // 285
  if (owner == edges.terminalEdge.sharer) {                                                           // 286
    return true;                                                                                      // 287
  }                                                                                                   // 288
                                                                                                      // 289
  var openerUserId = edges.terminalEdge.sharer;                                                       // 290
  var stackedUsers = {};                                                                              // 291
  stackedUsers[openerUserId] = true;                                                                  // 292
  var userStack = [openerUserId];                                                                     // 293
                                                                                                      // 294
  while (userStack.length > 0) {                                                                      // 295
    var recipient = userStack.pop();                                                                  // 296
    var edges = edgesByRecipient[recipient];                                                          // 297
    if (edges) {                                                                                      // 298
      for (var ii = 0; ii < edges.length; ++ii) {                                                     // 299
        var inEdge = edges[ii];                                                                       // 300
        var sharer = inEdge.sharer;                                                                   // 301
        if (sharer == owner) {                                                                        // 302
          return true;                                                                                // 303
        }                                                                                             // 304
        if (!stackedUsers[sharer]) {                                                                  // 305
          userStack.push(sharer);                                                                     // 306
          stackedUsers[sharer] = true;                                                                // 307
        }                                                                                             // 308
      }                                                                                               // 309
    }                                                                                                 // 310
  }                                                                                                   // 311
  return false;                                                                                       // 312
}                                                                                                     // 313
                                                                                                      // 314
SandstormPermissions.downstreamTokens = function(db, root) {                                          // 315
  // Computes a list of the UiView tokens that are downstream in the sharing graph from a given       // 316
  // source. The source, `root`, can either be a token or a (grain, user) pair. The exact format      // 317
  // of `root` is specified in the `check()` invocation below.                                        // 318
  //                                                                                                  // 319
  // TODO(someday): Once UiView tokens can have membrane requirements, we'll need to account for      // 320
  // them in this computation.                                                                        // 321
                                                                                                      // 322
  check(root, Match.OneOf({token: Match.ObjectIncluding({_id: String, grainId: String})},             // 323
                          {grain: Match.ObjectIncluding({_id: String, userId: String})}));            // 324
                                                                                                      // 325
  var result = [];                                                                                    // 326
  var tokenStack = [];                                                                                // 327
  var stackedTokens = {};                                                                             // 328
  var tokensBySharer = {};                                                                            // 329
  var tokensByParent = {};                                                                            // 330
  var tokensById = {};                                                                                // 331
                                                                                                      // 332
  function addChildren(tokenId) {                                                                     // 333
    var children = tokensByParent[tokenId];                                                           // 334
    if (children) {                                                                                   // 335
      children.forEach(function (child) {                                                             // 336
        if (!stackedTokens[child._id]) {                                                              // 337
          tokenStack.push(child);                                                                     // 338
          stackedTokens[child._id] = true;                                                            // 339
        }                                                                                             // 340
      });                                                                                             // 341
    }                                                                                                 // 342
  }                                                                                                   // 343
                                                                                                      // 344
  function addSharedTokens(sharer) {                                                                  // 345
    var sharedTokens = tokensBySharer[sharer];                                                        // 346
    if (sharedTokens) {                                                                               // 347
      sharedTokens.forEach(function (sharedToken) {                                                   // 348
        if (!stackedTokens[sharedToken._id]) {                                                        // 349
          tokenStack.push(sharedToken);                                                               // 350
          stackedTokens[sharedToken._id] = true;                                                      // 351
        }                                                                                             // 352
      });                                                                                             // 353
    }                                                                                                 // 354
  }                                                                                                   // 355
                                                                                                      // 356
  var grainId;                                                                                        // 357
  if (root.token) {                                                                                   // 358
    grainId = root.token.grainId;                                                                     // 359
  } else if (root.grain) {                                                                            // 360
    grainId = root.grain._id;                                                                         // 361
  }                                                                                                   // 362
                                                                                                      // 363
  var grain = db.getGrain(grainId);                                                                   // 364
  if (!grain || !grain.private ) { return result; }                                                   // 365
                                                                                                      // 366
  db.collections.apiTokens.find({grainId: grainId,                                                    // 367
                                 revoked: {$ne: true}}).forEach(function (token) {                    // 368
    tokensById[token._id] = token;                                                                    // 369
    if (token.parentToken) {                                                                          // 370
      if (!tokensByParent[token.parentToken]) {                                                       // 371
        tokensByParent[token.parentToken] = [];                                                       // 372
      }                                                                                               // 373
      tokensByParent[token.parentToken].push(token);                                                  // 374
    } else if (token.userId) {                                                                        // 375
      if (!tokensBySharer[token.userId]) {                                                            // 376
        tokensBySharer[token.userId] = [];                                                            // 377
      }                                                                                               // 378
      tokensBySharer[token.userId].push(token);                                                       // 379
    }                                                                                                 // 380
  });                                                                                                 // 381
                                                                                                      // 382
  if (root.token) {                                                                                   // 383
    addChildren(root.token._id);                                                                      // 384
  } else if (root.grain) {                                                                            // 385
    addSharedTokens(root.grain.userId);                                                               // 386
  }                                                                                                   // 387
                                                                                                      // 388
  while (tokenStack.length > 0) {                                                                     // 389
    var token = tokenStack.pop();                                                                     // 390
    result.push(token);                                                                               // 391
    addChildren(token._id);                                                                           // 392
    if (token.owner && token.owner.user) {                                                            // 393
      addSharedTokens(token.owner.user.userId);                                                       // 394
    }                                                                                                 // 395
  }                                                                                                   // 396
                                                                                                      // 397
  return result;                                                                                      // 398
}                                                                                                     // 399
                                                                                                      // 400
SandstormPermissions.createNewApiToken = function (db, userId, grainId, petname,                      // 401
                                                   roleAssignment, forSharing,                        // 402
                                                   expiresIfUnusedDuration,                           // 403
                                                   rawParentToken) {                                  // 404
  // Creates a new UiView API token. If `rawParentToken` is set, creates a child token.               // 405
  check(grainId, String);                                                                             // 406
  check(petname, String);                                                                             // 407
  check(roleAssignment, db.roleAssignmentPattern);                                                    // 408
  check(forSharing, Boolean);                                                                         // 409
  // Meteor bug #3877: we get null here instead of undefined when we                                  // 410
  // explicitly pass in undefined.                                                                    // 411
  check(expiresIfUnusedDuration, Match.OneOf(undefined, null, Number));                               // 412
  check(rawParentToken, Match.OneOf(undefined, null, String));                                        // 413
                                                                                                      // 414
  var parentToken;                                                                                    // 415
  if (rawParentToken) {                                                                               // 416
    parentToken = Crypto.createHash("sha256").update(rawParentToken).digest("base64");                // 417
  }                                                                                                   // 418
  if (!userId) {                                                                                      // 419
    if (!rawParentToken) {                                                                            // 420
      throw new Meteor.Error(403, "If you are not logged in, you must specify a parent token to " +   // 421
                             "create a new API token.");                                              // 422
    } else {                                                                                          // 423
      var parentApiToken = db.collections.apiTokens.findOne(                                          // 424
        {_id: parentToken, grainId: grainId, objectId: {$exists: false}});                            // 425
      if (!parentApiToken) {                                                                          // 426
        throw new Meteor.Error(403, "No such parent token found.");                                   // 427
      }                                                                                               // 428
      userId = parentApiToken.userId;                                                                 // 429
      if (parentApiToken.forSharing) {                                                                // 430
        forSharing = true;                                                                            // 431
      }                                                                                               // 432
    }                                                                                                 // 433
  }                                                                                                   // 434
                                                                                                      // 435
  var grain = db.getGrain(grainId);                                                                   // 436
  if (!grain) {                                                                                       // 437
    throw new Meteor.Error(403, "Unauthorized", "No grain found.");                                   // 438
  }                                                                                                   // 439
                                                                                                      // 440
  var token = Random.secret();                                                                        // 441
                                                                                                      // 442
  var apiToken = {                                                                                    // 443
    _id: Crypto.createHash("sha256").update(token).digest("base64"),                                  // 444
    userId: userId,                                                                                   // 445
    grainId: grainId,                                                                                 // 446
    roleAssignment: roleAssignment,                                                                   // 447
    petname: petname,                                                                                 // 448
    created: new Date(),                                                                              // 449
    expires: null,                                                                                    // 450
    forSharing: forSharing,                                                                           // 451
  };                                                                                                  // 452
                                                                                                      // 453
  if (parentToken) {                                                                                  // 454
    apiToken.parentToken = parentToken;                                                               // 455
  }                                                                                                   // 456
  if (expiresIfUnusedDuration) {                                                                      // 457
    apiToken.expiresIfUnused = new Date(Date.now() + expiresIfUnusedDuration);                        // 458
  }                                                                                                   // 459
                                                                                                      // 460
  db.collections.apiTokens.insert(apiToken);                                                          // 461
                                                                                                      // 462
  var endpointUrl = Url.parse(process.env.ROOT_URL).protocol + "//" + db.makeWildcardHost("api");     // 463
  return {id: apiToken._id, token: token, endpointUrl: endpointUrl};                                  // 464
}                                                                                                     // 465
                                                                                                      // 466
// Make self-destructing tokens actually self-destruct, so they don't                                 // 467
// clutter the token list view.                                                                       // 468
SandstormPermissions.cleanupSelfDestructing = function (db) {                                         // 469
  return function () {                                                                                // 470
    var now = new Date();                                                                             // 471
    db.collections.apiTokens.remove({expiresIfUnused: {$lt: now}});                                   // 472
  }                                                                                                   // 473
}                                                                                                     // 474
                                                                                                      // 475
Meteor.methods({                                                                                      // 476
  transitiveShares: function(grainId) {                                                               // 477
    check(grainId, String);                                                                           // 478
    if (this.userId) {                                                                                // 479
      var db = this.connection.sandstormDb;                                                           // 480
      return SandstormPermissions.downstreamTokens(db,                                                // 481
          {grain: {_id: grainId, userId: this.userId}});                                              // 482
    }                                                                                                 // 483
  },                                                                                                  // 484
                                                                                                      // 485
  newApiToken: function (grainId, petname, roleAssignment, forSharing,                                // 486
                         expiresIfUnusedDuration, rawParentToken) {                                   // 487
    return SandstormPermissions.createNewApiToken(                                                    // 488
      this.connection.sandstormDb, this.userId, grainId, petname, roleAssignment, forSharing,         // 489
      expiresIfUnusedDuration, rawParentToken);                                                       // 490
  },                                                                                                  // 491
                                                                                                      // 492
  updateApiToken: function (token, newFields) {                                                       // 493
    var db = this.connection.sandstormDb;                                                             // 494
                                                                                                      // 495
    check(token, String);                                                                             // 496
    check(newFields, {petname: Match.Optional(String),                                                // 497
                      roleAssignment: Match.Optional(db.roleAssignmentPattern),                       // 498
                      revoked: Match.Optional(Boolean)});                                             // 499
                                                                                                      // 500
    var apiToken = db.collections.apiTokens.findOne(token);                                           // 501
    if (!apiToken) {                                                                                  // 502
      throw new Meteor.Error(404, "No such token found.");                                            // 503
    }                                                                                                 // 504
    if (this.userId && (apiToken.userId === this.userId)) {                                           // 505
      var modifier = {$set: newFields};                                                               // 506
      db.collections.apiTokens.update(token, modifier);                                               // 507
    } else {                                                                                          // 508
      throw new Meteor.Error(403, "User not authorized to modify this token.");                       // 509
    }                                                                                                 // 510
  }                                                                                                   // 511
});                                                                                                   // 512
                                                                                                      // 513
////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-permissions'] = {
  SandstormPermissions: SandstormPermissions
};

})();

//# sourceMappingURL=sandstorm-permissions.js.map
